#include "POV.h"
POV::POV(byte clock, byte latch, byte data)
{
	clockPin = clock;
	latchPin = latch;
	dataPin = data;
	pinMode(clockPin, OUTPUT);
	pinMode(latchPin, OUTPUT);
	pinMode(dataPin, OUTPUT);
	// Initiate spoke objects
	spokes[0].init(A_SENSOR, A_L_ENABLE, A_R_ENABLE);
	spokes[1].init(B_SENSOR, B_L_ENABLE, B_R_ENABLE);
	spokes[2].init(C_SENSOR, C_L_ENABLE, C_R_ENABLE);
	nextSensor = 1;
	for(int i = 0; i < SPOKES; i++)
	{
		currentFrame[i] = i * FRAME_GROUP;
	}
}

// Scroll some bits without regard to timing. Useful for standing still.
bool POV::idle()
{
#ifdef DEBUG
	Serial.println("idle()");
#endif
	uint8_t pattern[4] = {0x11, 0x22, 0x44, 0x88};

	for(int i = 0; i < SPOKES; i++)
	{
		setEnableByte(0x3, i);
	}
	for(int i = 0; i < 4; i++)
	{
		for(int j = 0; j < 9; j++)
		{
			if(readSensors())
			{
				return true;
			}
			else
			{
				shiftOneByte(pattern[i]);
				delay(20);
			}
		}
	}
	return false;
}

// get timing data from sensors
bool POV::readSensors()
{
#ifdef DEBUG
	Serial.print("readSensors()");
#endif
	// Update timing if the sensor reads true.
	if(spokes[nextSensor].readSensor())
	{
		// Sync current frame to current wheel position
		setCurrentFrame((FRAME_GROUP) * nextSensor);

		// Update the next sensor since we don't need to read this one again.
		if(nextSensor == 2) nextSensor = 0;
		else nextSensor++;

#ifdef DEBUG
		Serial.println("  true");
#endif
		return true;
	}
	else
	{
#ifdef DEBUG
		Serial.println("  false");
#endif
		return false;
	}
}

void POV::setCurrentFrame(uint8_t aFrame)
{
#ifdef DEBUG
	Serial.print("setCurrentFrame(");
	Serial.print(aFrame);
	Serial.println(")");
#endif
	// Arbitrarily set the current frame
	for(int i = 0; i < SPOKES; i++)
	{
		currentFrame[i] = aFrame + i * FRAME_GROUP;
		if(currentFrame[i] >= FRAMES) currentFrame[i] -= FRAMES;
	}
}

void POV::advanceCurrentFrame()
{
#ifdef DEBUG
	Serial.println("advanceCurrentFrame()");
#endif
	for(int i = 0; i < SPOKES; i++)
	{
		currentFrame[i]++;
		if (currentFrame[i] >= FRAMES) currentFrame[i] -= FRAMES;
	}
}

// Send all three spokes and latch.
void POV::display(imageClass *image)
{
#ifdef DEBUG
	Serial.println("display()");
#endif
	// Clear old enable value
	enableByte = 0x0;

	/*
	Send each spoke down the data line and set enable bits. This happens in
	reverse to get the last spoke's data to the last spoke.
	*/
	for(int i = SPOKES - 1; i >= 0; i--)
	{
		setupSingleSpoke(image, i);
	}
	latch();
}

// Get a spoke ready for display
void POV::setupSingleSpoke(imageClass *image, uint8_t spoke)
{
#ifdef DEBUG
	Serial.print("setupSingleSopke(");
	Serial.print(spoke);
	Serial.println(")");
#endif
	// Send three bytes for a single spoke. Outer byte first.
	for(int i = SPOKES - 1; i >= 0; i--)
	{
		shiftOneByte(image->getPixels(currentFrame[spoke], i));
	}
	// Enable or disable the left and right side.
	setEnableByte(image->getEnable(currentFrame[spoke]), spoke);
}

// Send each bit of the byte down the data wire with a clock tick
void POV::shiftOneByte(uint8_t data)
{
#ifdef DEBUG
	Serial.print("shiftOneByte(");
	Serial.print(data);
	Serial.println(")");
#endif
	// the last bit is sent first because it needs to reach the last register
	for (int bit = 7; bit >= 0; bit--)
	{
		// set the clock up
		digitalWrite(clockPin, 0);

		// compare 1 and the value of the bit in the byte.
		if (data & (1 << bit))
		{
			digitalWrite(dataPin, HIGH);
		}
		else
		{
			digitalWrite(dataPin, LOW);
		}

		// The falling edge of the clock signal sets the data into the shift
		// register memory.
		digitalWrite(clockPin, 1);
	}
}

// Directly setup the port for precise timing
void POV::setEnableByte(uint8_t data, uint8_t spoke)
{
#ifdef DEBUG
	Serial.print("setEnableByte(");
	Serial.print(data);
	Serial.print(",");
	Serial.print(spoke);
	Serial.println(")");
#endif
	// Shift the bits
	data <<= (spoke * 2) + 2;
	// Set the bits
	enableByte |= data;
}

// Send the latch signal
void POV::latch()
{
#ifdef DEBUG
	Serial.println("latch()");
#endif
	digitalWrite(latchPin, HIGH);
	PORTD = enableByte;
	digitalWrite(latchPin, LOW);
}